metrics = [
    "Aboderin",
    "AbrahamLeo",
    "Argos",
    "Barley",
    "BlackMould",
    "BullBreese",
    "Casari",
    "Chothia",
    "Cid",
    "Cowan3.4",
    "Cowan7.5",
    "Eisenberg",
    "Engelman",
    "Fasman",
    "Fauchere",
    "Goldsack",
    "Guy",
    "HoppWoods",
    "Janin",
    "Jones",
    "Juretic",
    "Kidera",
    "Kuhn",
    "KyteDoolittle",
    "Levitt",
    "Manavalan",
    "Miyazawa",
    "Parker",
    "Ponnuswamy",
    "Prabhakaran",
    "Rao",
    "Rose",
    "Roseman",
    "Sweet",
    "Tanford",
    "Welling",
    "Wilson",
    "Wolfenden",
    "Zimmerman",
    "interfaceScale_pH2",
    "interfaceScale_pH8",
    "octanolScale_pH2",
    "octanolScale_pH8",
    "oiScale_pH2",
    "oiScale_pH8"
]